%% EISCaseStudy_Main.m
% This file is the main script to be used for the MATLAB companion tutorial
% for the document titled "EIS Model Fitting and Analysis". 
%
% Note: requires EIS_DATA.mat, Objective.m, Optimize.m, RRCRC_MDL.m,
% and RZARCCPE_MDL.m in same folder as this script. 
%
% Author: Marvin Messing, CMHT, McMaster University
% Date: July 26, 2017

%% LOAD DATA
load("EIS_DATA.mat")


% variable to hold Ohm-mOhm conversion
to_mOhm = 1000;
to_Hz = 1000;

% setup model parameter solutions and initialization
model_parameters_RZARCCPE = [5.31e-07,0.022,0.00509,0.717,0.76,393.14,0.62];
model_parameters_RZARCCPE_init = [1e-06, 0.0236, 0.0036, 1e3, 1, 1e3, 1]; 

model_parameters_RRC = [4.231e-7, 0.0232, 0.0038, 0.2583, 0.0022, 173.41];
model_parameters_RRC_init = [1e-06, 0.0236, 0.0036, 1e6, 1, 1e6];


%% MODEL FITTING
%% MODEL 1: R-RC. Uncomment the floowing lines to parameterize this model. 

MDL_COL = 'k';
MDL_NAME = "R-RC-RC";
MDL = @RRCRC_MDL;
model_parameters = model_parameters_RRC_init;
ub = [1e-06, 0.024, 0.0060, 1e3,     1, 1e3];
lb = [1e-08, 0.022, 0.0036, 1e-4, 1e-4, 1e-4];

%% MODEL 2: R-ZARC-CPE. Uncomment the following lines to parameterize this model. 

MDL_COL = 'm';
MDL_NAME = "R-ZARC-CPE";
MDL = @RZARCCPE_MDL;
model_parameters = model_parameters_RZARCCPE_init;
ub = [1e-06, 0.024, 0.0060, 1e3, 1, 1e3, 1];
lb = [1e-08, 0.022, 0.0036,   0, 0,   0, 0];

%% RUN OPTIMIZATION

% provide arguments for the objective function to be used in the
% optimization

% Prepare a handle to the objective function with the model specific input parameters. 
fun = @(model_parameters)Objective(model_parameters, MDL, EIS_DATA);
% Call the opimiziation method. The last argument determines the type of algorithm (1-least squares, 2-PSO). 
x = Optimize(fun, model_parameters, lb, ub, 2);

%% PLOT NYQUIST

%%% Collect frequency labels 
lbl_idx = [1, 8, 20, 31];
dy = [0.1, 0.1, -0.1, -0.1];

for j = 1:length(lbl_idx)
   i = lbl_idx(j);
   lbl_re(j) = EIS_DATA.Real(i)*to_mOhm+0.1;
   lbl_img(j) = EIS_DATA.Imaginary(i)*to_mOhm+dy(j);
   lbl_text{j} = num2str(round(EIS_DATA.Frequency(i)*to_Hz,1)) + " Hz";
end

%%% PLOT
figure(1)
hold on
scatter(EIS_DATA.Real*to_mOhm, EIS_DATA.Imaginary*to_mOhm)

text(lbl_re, lbl_img, lbl_text);

xlabel("Z' / {m\Omega}")
ylabel("-Z'' / {m\Omega}")

legend("EIS Data, 100% SoC, 25" + char(176) + "C, 3 hr rest", "location", "southeast")
grid on

%% PLOT NYQUIST FIT RESULT

figure(1);
% Get Z from latest fitting result
Z = MDL(EIS_DATA.Frequency*to_Hz, x);
S = Objective(x, MDL, EIS_DATA);

% Get Z from initialized model parameters
%Z = MDL(EIS_DATA.Frequency*to_Hz, model_parameters);

plot(real(Z)*to_mOhm, -1*imag(Z)*to_mOhm, 'DisplayName',MDL_NAME, 'Color', MDL_COL);

%% PLOT BODE

% Calcilate Z (magnitude) from real and imaginary data
Zmag = abs(EIS_DATA.Real*to_mOhm + 1j*EIS_DATA.Imaginary*to_mOhm);

figure(2)
ax1 = subplot(2,1,1);
loglog(EIS_DATA.Frequency*to_Hz, Zmag)
ylabel("log |Z / {m\Omega}|")
grid on

ax2 = subplot(2,1,2);
semilogx(EIS_DATA.Frequency*to_Hz, rad2deg(EIS_DATA.Phase))
ylabel("{\phi} / deg")

xlabel("log (f / {s^{-1}})")
grid on

legend("EIS Data, 100% SoC, 25" + char(176) + "C, 3 hr rest", "location", "best")

%% PLOT BODE FIT RESULT

% Get Z from latest fitting result
Z = MDL(EIS_DATA.Frequency*to_Hz, x);

% Get Z from initialized model parameters
%Z = MDL(EIS_DATA.Frequency*to_Hz, model_parameters);

% Calcilate phase from Z
PHS = rad2deg(atan(imag(Z)./real(Z)));

axes(ax1)
hold on
loglog(EIS_DATA.Frequency*to_Hz, abs(Z)*to_mOhm, 'DisplayName',MDL_NAME, 'Color', MDL_COL);

axes(ax2)
hold on
semilogx(EIS_DATA.Frequency*to_Hz, PHS, 'DisplayName',MDL_NAME, 'Color', MDL_COL);

